SystemOrganization addCategory: #'Volunti-CertificationAuthority-Model'!
SystemOrganization addCategory: #'Volunti-CertificationAuthority-View'!
SystemOrganization addCategory: #'Volunti-CertificationAuthority-Tests'!

TestCase subclass: #VOCertificationAuthTest
	instanceVariableNames: 'certificationAuth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-CertificationAuthority-Tests'!

!VOCertificationAuthTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 10/29/2012 16:33'!
setUp
	certificationAuth := VOCertificationAuthority new.! !

!VOCertificationAuthTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 1/4/2012 09:56'!
testCreateCertificate
	certificationAuth certificate.! !

VOCertifiedAgent subclass: #VOCertificationAuthority
	instanceVariableNames: 'certificates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-CertificationAuthority-Model'!

!VOCertificationAuthority class methodsFor: 'x509' stamp: 'NicolasVanderAuwera 12/11/2011 14:08'!
organisationUnit
	^ 'Certification Authority'! !

!VOCertificationAuthority class methodsFor: 'network' stamp: 'NicolasVanderAuwera 12/1/2011 09:27'!
port
	^ 6754! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 1/4/2012 10:15'!
addCommonFieldsIn: tbsCertificate
	self addVersionIn: tbsCertificate.
	self addSerialNumberIn: tbsCertificate.
	self addIssuerIn: tbsCertificate.
	self addValidityIn: tbsCertificate! !

!VOCertificationAuthority methodsFor: 'adding' stamp: 'NicolasVanderAuwera 1/4/2012 11:17'!
addIssuerIn: tbsCertificate
	tbsCertificate issuer: self subjectInfo dn! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 1/4/2012 10:14'!
addSerialNumberIn: tbsCertificate
	tbsCertificate serialNumber: self serialNumber! !

!VOCertificationAuthority methodsFor: 'adding' stamp: 'NicolasVanderAuwera 1/10/2012 07:52'!
addSignatureIn: aCertificate
	aCertificate signatureAlgorithm: (X509AlgorithmIdentifier oidString: '1.2.840.113549.1.1.5').
	aCertificate tbsCertificate signature: aCertificate signatureAlgorithm.
	aCertificate signature: (self sign: aCertificate tbsCertificate asStreamedObject)! !

!VOCertificationAuthority methodsFor: 'adding' stamp: 'NicolasVanderAuwera 12/30/2011 16:45'!
addValidityIn: tbsCertificate
	| date |
	date := DateAndTime now asDate.
	tbsCertificate
		validity:
			(X509Validity new
				notBefore: date;
				notAfter: (date := date addMonths: 12);	"1 year of validity"
				yourself)! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 1/4/2012 10:13'!
addVersionIn: tbsCertificate
	tbsCertificate version: self defaultVersion! !

!VOCertificationAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 11/1/2012 08:23'!
certificate
	certificate isNil
		ifTrue: [ self createMyCertificate ].
	^ certificate! !

!VOCertificationAuthority methodsFor: 'searching' stamp: 'NicolasVanderAuwera 8/2/2012 15:07'!
certificateWithSubjectInfo: aSubjectInfo
	^ self certificates detect: [ :cert | cert tbsCertificate subject = aSubjectInfo ] ifNone: [ nil ]! !

!VOCertificationAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/11/2012 08:04'!
certificates
	^ certificates ifNil: [ certificates := Set new ]! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 11/20/2012 14:19'!
createCertificate: subject
	| oldCertificate newCertificate |
	newCertificate := X509Certificate new.
	newCertificate tbsCertificate: (self createTBSCertificate: subject).
	self addSignatureIn: newCertificate.
	stream sendObject: newCertificate.
	oldCertificate := self certificateWithSubjectInfo: newCertificate tbsCertificate subject.
	oldCertificate ifNotNil: [ self certificates remove: oldCertificate ].
	self certificates add: newCertificate! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 11/1/2012 08:23'!
createMyCertificate
	certificate := X509Certificate new.
	certificate tbsCertificate: self createMyTBSCertificate.
	self addSignatureIn: certificate.
	self certificates add: certificate! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 5/6/2012 10:16'!
createMyTBSCertificate
	| tbsCertificate |
	tbsCertificate := X509TBSCertificate new.
	self addCommonFieldsIn: tbsCertificate.
	tbsCertificate subject: self subjectInfo.
	tbsCertificate subjectPublicKeyInfo: self subjectKeyInfo.
	^ tbsCertificate! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 11/20/2012 14:09'!
createTBSCertificate: subject
	| tbsCertificate |
	tbsCertificate := X509TBSCertificate new.
	self addCommonFieldsIn: tbsCertificate.
	tbsCertificate subject: (subject at: #subjectInfo).
	tbsCertificate subjectPublicKeyInfo: (subject at: #subjectKeyInfo).
	^ tbsCertificate! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 1/4/2012 10:13'!
defaultVersion
	^ 3! !

!VOCertificationAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/20/2012 14:05'!
handle: aMessage
	super handle: aMessage.
	aMessage command = #registration
		ifTrue: [ self createCertificate: aMessage content ].
	aMessage command = #certificateOf
		ifFalse: [ ^ self ].
	self sendCertificate! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 11/1/2012 08:23'!
refreshCertificate
	self createMyCertificate! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 12/31/2011 11:16'!
requestCertificate
	^ self certificate
	! !

!VOCertificationAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 11/1/2012 14:15'!
requestKeys
	| gen |
	gen := VORSAKeyPairGenerator new bits: CryptographicParameters rsaKeyLength.
	publicKey := gen publicKey.
	privateKey := gen privateKey! !

!VOCertificationAuthority methodsFor: 'network' stamp: 'NicolasVanderAuwera 10/29/2012 14:56'!
sendCertificate
	| remoteSubjectInfo remoteCertificate |
	remoteSubjectInfo := stream receiveObject.
	remoteCertificate := self certificateWithSubjectInfo: remoteSubjectInfo.
	stream sendObject: remoteCertificate.
	self addInLog: 'A new certificate has been sent to ' , remoteSubjectInfo organisationUnit! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 1/4/2012 10:14'!
serialNumber
	^ 123456789! !

!VOCertificationAuthority methodsFor: 'cryptography' stamp: 'NicolasVanderAuwera 3/5/2012 15:37'!
sign: msg
	^ self publicKey v15SignMessage: msg
	! !

VOAgentComponent subclass: #VOCertificationAuthorityComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-CertificationAuthority-View'!

!VOCertificationAuthorityComponent class methodsFor: 'class initialization' stamp: 'NicolasVanderAuwera 12/16/2012 09:56'!
initialize
	self register! !

!VOCertificationAuthorityComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/10/2011 09:24'!
url
	^ 'certificationauth'! !

!VOCertificationAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 10/29/2012 16:33'!
agentClass
	^ VOCertificationAuthority! !

!VOCertificationAuthorityComponent methodsFor: 'actions' stamp: 'NicolasVanderAuwera 10/29/2012 15:23'!
setHome
	content := (VOCertificateListComponent list: self agent certificates)
		addDecoration: VOErrorHandlingDecoration new;
		yourself! !

!VOCertificationAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 12/10/2011 09:25'!
title
	^ 'Certification Authority'! !

WAComponent subclass: #VOCertificateListComponent
	instanceVariableNames: 'list'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-CertificationAuthority-View'!

!VOCertificateListComponent class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 10/29/2012 15:20'!
list: aCollection
	^ self new list: aCollection! !

!VOCertificateListComponent methodsFor: 'actions' stamp: 'NicolasVanderAuwera 10/29/2012 16:00'!
downloadCertificate: certificate
	| file |
	file := WAFile new
		contentType: WAMimeType textPlain;
		contents: certificate xmlDescription;
		fileName: 'certificate';
		yourself.
	self requestContext
		respond: [ :response | 
			response
				contentType: 'text/xml';
				document: file rawContents greaseString;
				attachmentWithFileName: file fileName ]! !

!VOCertificateListComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 10/29/2012 15:21'!
list
	^ list! !

!VOCertificateListComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 10/29/2012 15:21'!
list: anObject
	list := anObject! !

!VOCertificateListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 11/1/2012 09:00'!
renderCertificate: certificate on: html
	^ html
		tableRow: [ 
			html
				tableData: [ certificate tbsCertificate subject dn do: [ :part | html text: part ] separatedBy: [ html text: ', ' ] ].
			html
				tableData: [ 
					html span
						class: 'label label-success' if: (certificate isValid and: [ certificate isAlmost not ]);
						class: 'label label-warning' if: certificate isAlmost;
						class: 'label label-important' if: certificate isPassed;
						with: certificate tbsCertificate validity notAfter ].
			html
				tableData: [ 
					html anchor
						class: 'btn btn-info btn-small';
						callback: [ self downloadCertificate: certificate ];
						with: [ 
									html span class: 'icon-download'.
									html text: ' Download' ] ] ]! !

!VOCertificateListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/29/2012 15:45'!
renderCertificatesOn: html
	html table
		class: 'table';
		with: [ 
					self renderHeadingOn: html.
					self list do: [ :certificate | self renderCertificate: certificate on: html ] ]! !

!VOCertificateListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/29/2012 16:19'!
renderContentOn: html
	self renderTitleOn: html.
	self renderLegendOn: html.
	self renderCertificatesOn: html! !

!VOCertificateListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/29/2012 16:01'!
renderHeadingOn: html
	html
		tableRow: [ 
			html tableHeading: 'Distinguished name'.
			html tableHeading: 'Valid until'.
			html tableHeading: '' ]! !

!VOCertificateListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/29/2012 16:29'!
renderLegendOn: html
	html
		paragraph: [ 
			html span
				class: 'label label-success';
				with: 'date'.
			html text: ' The certificate is valid; '.
			html span
				class: 'label label-warning';
				with: 'date'.
			html text: ' The certificate is obsolete in less than ' , X509Certificate almostDays greaseString , ' days; '.
			html span
				class: 'label label-important';
				with: 'date'.
			html text: ' The certificate is obsolete.' ]! !

!VOCertificateListComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 10/29/2012 15:26'!
renderTitleOn: html
	html heading
		level3;
		with: self title! !

!VOCertificateListComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 10/29/2012 15:24'!
title
	^ 'List of certificates'! !
VOCertificationAuthorityComponent initialize!
